import os


class Vdir(object):
    """Simple vdir storage object to list and get items from a given vdir."""
    def __init__(self, path):
        """Initialises new Vdir object for a given path"""
        if os.path.isdir(path) is False:
            raise IOError(path)
        self.path = path

    def list(self):
        """Yields a list of all ics files in the vdir."""
        for filename in os.listdir(self.path):
            if filename.endswith(".ics"):
                yield filename

    def get(self, filename):
        """Gets contents of a given filename."""
        filepath = os.path.join(self.path, filename)
        with open(filepath, 'rb') as item:
            return item.read().decode("utf-8")



import jinja2
import logging

from xhtml2pdf import pisa
from cliff.command import Command
from tracklr import Tracklr
from jinja2 import Template
from jinja2.exceptions import TemplateNotFound


class Pdf(Command):

    log = logging.getLogger(__name__)

    tracklr = Tracklr()

    __template__ = """
<html>
  <header>
    <title>{{ tracklr.title }} {{ tracklr.subtitle }}</title>
    <style>
    table {
        font: 12px Verdana, Arial, Helvetica, sans-serif;
        width: 1200px;
    }

    th {
        padding: 0.5em 0.5em 0;
        border-bottom: 3px solid #555;
    }
    th.date {
        text-align: left;
        width: 100px;
    }
    th.summary {
        text-align: left;
        width: 1000px;
    }
    th.hours {
        text-align: right;
        width: 100px;
    }

    td {
        border-bottom: 1px solid #CCC;
        padding: 0.25em 0.5em 0;
        vertical-align: top;
    }
    td.tag {
        text-align: left;
        width: 100px;
    }
    td.date {
        text-align: left;
        width: 100px;
    }
    td.summary {
        text-align: left;
        width: 1000px;
    }
    td.hours {
        text-align: right;
        width: 100px;
    }
    tr.total {
        border-top: 2px solid #555;
        font-weight: bold;
    }
    </style>
  </header>
  <body>
    <h1>{{ tracklr.title }} - {{ tracklr.subtitle }}</h1>
    <table>
      {% if tracklr.tags %}
      <tr>
        <th class="date">Tag</th>
        <th class="hours">Hours</th>
      </tr>
        {% for item in tracklr.report_html %}
      <tr>
        <td class="tag">{{ item.0 }}</td>
        <td class="hours">{{ item.1 }}</td>
      </tr>
        {% endfor %}
      <tr class="total">
        <td>Total</td>
        <td class="hours">{{ tracklr.total_hours }}</td>
      </tr>
      {% else %}
      <tr>
        <th class="date">Date</th>
        <th class="summary">Summary</th>
        <th class="hours">Hours</th>
      </tr>
        {% for item in tracklr.report_html %}
      <tr>
        <td class="date">{{ item.0 }}</td>
        <td class="summary">
        {% if item.2 != "" %}
          {{ item.2 }}
        {% endif %}
        {% if item.4 not in item.2 %}
          {% if item.2 != "" %}
          <br />
          {% endif %}
          {{ item.4 }}
        {% endif %}
        </td>
        <td class="hours">{{ item.3 }}</td>
      </tr>
        {% endfor %}
      <tr class="total">
        <td colspan="2">Total</td>
        <td class="hours">{{ tracklr.total_hours }}</td>
      </tr>
      {% endif %}
    </table>
  </body>
</html>
        """

    def take_action(self, parsed_args):
        """Generates report as a PDF file.
        """
        cal = self.tracklr.get_calendar_config(parsed_args.kalendar)
        self.tracklr.get_calendar(cal["name"])

        if parsed_args.report == "tag":
            self.tracklr.get_tags(
                parsed_args.kalendar,
                parsed_args.date,
                parsed_args.include,
                parsed_args.exclude,
            )
        else:
            self.tracklr.get_report(
                parsed_args.kalendar,
                parsed_args.date,
                parsed_args.include,
                parsed_args.exclude,
            )

        in_html = self.generate_html()

        out_pdf = self.tracklr.get_pdf_output_file(parsed_args.file)

        self.tracklr.banner(
            parsed_args.kalendar,
            parsed_args.title,
            parsed_args.subtitle
        )

        self.log.info("\nOutput file: {}\n".format(out_pdf))

        self.generate_pdf(in_html, out_pdf)

    def get_description(self):
        """creates PDF report"""
        return "creates PDF report"

    def get_parser(self, prog_name):
        """Defines the following input arguments for ``pdf`` command:

        * ``-f`` ``--file`` destination of the pdf file
        * ``-r`` ``--report`` {ls, tag} pdf of ``ls`` (default) or ``tag``
        """
        parser = super(Pdf, self).get_parser(prog_name)
        parser.add_argument("-f", "--file")
        parser.add_argument("-r", "--report", type=str, choices=["ls", "tag"])
        return self.tracklr.get_parser(parser)

    def generate_html(self):
        """Generates HTML version"""
        try:
            loader = jinja2.FileSystemLoader(
                searchpath=self.tracklr.template_path
            )
            env = jinja2.Environment(loader=loader)
            template = env.get_template(self.tracklr.pdf_template_file)
            self.log.info("Template: {}".format(self.tracklr.pdf_template_file))
            return template.render(tracklr=self.tracklr)
        except TemplateNotFound:
            template = Template(self.__template__)
            self.log.info("Template: internal")
            return template.render(tracklr=self.tracklr)

    def generate_pdf(self, in_html, out_pdf):
        """Generates PDF from HTML version"""
        report_file = open(out_pdf, "w+b")
        status = pisa.CreatePDF(in_html, dest=report_file)
        report_file.close()



import logging

from cliff.lister import Lister
from tracklr import Tracklr


class Ls(Lister):

    log = logging.getLogger(__name__)

    tracklr = Tracklr()

    def take_action(self, parsed_args):
        """Generates report and logs total number of hours."""
        ts = self.tracklr.get_report(
            parsed_args.kalendar,
            parsed_args.date,
            parsed_args.include,
            parsed_args.exclude,
        )
        self.tracklr.banner(
            parsed_args.kalendar,
            parsed_args.title,
            parsed_args.subtitle
        )
        self.log.info("\nTotal hours: {}".format(self.tracklr.total_hours))
        self.log.info("Number of events: {}\n".format(len(ts)))

        return (("Date", "Summary", "Description", "Hours"), ts)

    def get_description(self):
        """Returns command description"""
        return "creates report"

    def get_parser(self, prog_name):
        """Gets default parser ie. this command does not add any new args"""
        parser = super(Ls, self).get_parser(prog_name)
        return self.tracklr.get_parser(parser)



# -*- coding: utf-8 -*-
import os
import re
import warnings

# Python2 Backward Compatibility
try:
    FileNotFoundError
except NameError:
    FileNotFoundError = IOError

# Python2 Backward Compatibility
try:
    ModuleNotFoundError
except NameError:
    ModuleNotFoundError = ImportError

try:
    import appdirs
except ModuleNotFoundError:
    print(
        "appdirs missing - should the issue persists post install, "
        "run `pip install appdirs` manually"
    )
import logging
import pytz
import requests
import yaml

from pyfiglet import Figlet
from icalendar import Calendar
from requests.auth import HTTPBasicAuth
from requests.exceptions import MissingSchema

from tracklr.vdir import Vdir


WARN_CONFIG_URL = ("Use `location` instead of `url` in your configuration. "
                   "`url` is deprecated since v1.0.0. "
                   "See https://www.tracklr.com/configuration.html")
WARN_CONFIG_TZ = ("Use `timezone` instead of `x_wr_timezone` "
                  "in your configuration. "
                  "`x_wr_timezone` is deprecated since v1.0.0. "
                  "See https://www.tracklr.com/configuration.html")


class Tracklr(object):
    """Tracklr loads events recorded in `iCalendar` feeds and
    uses them to create reports.
    """

    __version__ = "1.1.0"

    __config__ = """
---
# Tracklr Instance Log Level (debug, info, warning, error, critical)
log_level: info

# List of calendars
#
# calendar attributes:
# * location          - mandatory - ical calendar feed location - either URL or directory
# * name              - optional  - use `default` for your default calendar ie. useful for single calendar users
# * title/subtitle    - optional  - info used by `ls` and `pdf` commands
# * username/password - optional  - for BasicHTTPAuth protected feeds
#
calendars:
  # Tracklr demo calendar - simplest single calendar config v1
  #- https://calendar.google.com/calendar/ical/bdtrpfi80dtav668iqd38oqi7g%40group.calendar.google.com/public/basic.ics

  # Tracklr demo calendar - simplest single calendar config v2
  #- location: https://calendar.google.com/calendar/ical/bdtrpfi80dtav668iqd38oqi7g%40group.calendar.google.com/public/basic.ics

  # Tracklr demo calendar - simplest single calendar config v3
  # X-WR-TIMEZONE support for of the demo Google Calendar feed enabled
  # This fixes timezone issue because the demo calendar is in New Zealand timezone and reports
  # would be showing incorrect dates ie.
  # | 2019-03-29 - 2019-03-30 | @Tracklr #v0.7
  # instead of correct
  # | 2019-03-30 | @Tracklr #v0.7
  - location: https://calendar.google.com/calendar/ical/bdtrpfi80dtav668iqd38oqi7g%40group.calendar.google.com/public/basic.ics
    title: Tracklr
    subtitle: Demo Calendar
    timezone: True

  # Tracklr demo calendar - minimal default config
  #- name: minimal
  #  location: https://calendar.google.com/calendar/ical/bdtrpfi80dtav668iqd38oqi7g%40group.calendar.google.com/public/basic.ics

  # Tracklr demo calendar - full config
  #- name: full
  #  location: https://calendar.google.com/calendar/ical/bdtrpfi80dtav668iqd38oqi7g%40group.calendar.google.com/public/basic.ics
  #  title: Tracklr Demo
  #  subtitle: Report
  #  timezone: Pacific/Auckland

  # Example of vdir configuration
  #- name: demo
  #  location: ~/.calendars/ab14901f-017b-78df-28bc-92d9387e5cfb

  # 
  #- name: 
  #  location: 
  #  username: 
  #  password: 
        """

    def __init__(self):
        """Initializes Tracklr object with its configuration.
        """
        self.log_levels = {
            "DEBUG": logging.DEBUG,
            "INFO": logging.INFO,
            "WARNING": logging.WARNING,
            "ERROR": logging.ERROR,
            "CRITICAL": logging.CRITICAL,
        }

        self.calendars = dict()

        self.report = []
        self.report_html = []

        self.tags = dict()

        self.total_seconds = 0.0
        self.total_hours = 0.0

        self.pdf_template_file = "pdf.html"
        self.pdf_output_file = "report.pdf"

        self.local_path = os.getcwd()
        self.global_path = os.path.join(appdirs.user_config_dir(), "tracklr")

        self.template_path = [self.local_path, self.global_path]

        self.config_file = "tracklr.yml"
        self.config_dot_file = ".tracklr.yml"

        self.loaded_config_file = None

        self.config = None
        self.configure()

    def banner(self, kalendar, title=None, subtitle=None):
        """Displays core information about the Tracklr instance.
        """
        cal = self.get_calendar_config(kalendar)

        title = self.get_title(cal["name"], title)
        subtitle= self.get_subtitle(cal["name"], subtitle)

        banner = Figlet(font="cybermedium")
        self.log.info("{} v{}\n".format(
            banner.renderText("Tracklr").rstrip(" \n"),
            self.__version__)
        )

        self.log.info("Title: {}".format(title))
        self.log.info("Subtitle: {}".format(subtitle))
        self.log.info("Configuration: {}".format(self.loaded_config_file))

    def configure(self):
        """Tries to load Tracklr configuration from current working directory
        then user config directory and if none found it defaults to internal
        configuration stored in ``Tracklr.__config__``.

        Once config loaded, processes ``calendars`` list from the config and
        handles various configuration options.
        """

        def loadrc(config_file):
            self.config = yaml.safe_load(open(config_file, "r"))
            self.loaded_config_file = config_file

        try:
            loadrc(self.config_dot_file)
        except FileNotFoundError:
            try:
                loadrc(self.config_file)
            except FileNotFoundError:
                try:
                    loadrc(
                        os.path.join(self.global_path, self.config_dot_file)
                    )
                except FileNotFoundError:
                    try:
                        loadrc(
                            os.path.join(self.global_path, self.config_file)
                        )
                    except FileNotFoundError:
                        self.config = yaml.safe_load(self.__config__)
                        self.loaded_config_file = "default"

        self.get_logger()

        for cal in self.config["calendars"]:
            try:
                name = cal["name"]
                self.calendars[name] = cal
            except KeyError:
                name = "default"
                try:
                    self.calendars[name] = {
                        "name": name,
                        "location": cal["location"],
                    }
                except KeyError:
                    self.calendars[name] = {
                        "name": name,
                        "location": cal["url"],
                    }
                    warnings.warn(WARN_CONFIG_URL)
            except TypeError:
                name = "default"
                self.calendars[name] = {"name": name, "location": cal}

    def get_calendar_config(self, calendar):
        """Returns given calendar config or
        raises exception if none found.
        """
        if not calendar:
            calendar = "default"
        calendars = "\n".join([cal for cal in self.calendars])
        if calendar not in self.calendars:
            self.log.error(
                "calendar {} not found in configured calendars:\n{}".format(
                    calendar, calendars
                )
            )
            raise
        return self.calendars[calendar]

    def get_logger(self):
        """Sets up logger
        """
        logging.basicConfig()

        self.log = logging.getLogger(__name__)

        log_level = self.log_levels["INFO"]
        if "log_level" in self.config:
            log_level = self.log_levels[self.config["log_level"].upper()]
            self.log.debug(
                "Set logging to {}".format(self.config["log_level"].upper())
            )

        self.log.setLevel(log_level)

    def get_title(self, calendar, title):
        """Handles title of the provided calendar.

        Title is optional in the configuration so default title is "Tracklr".
        """
        if "dir" in self.calendars[calendar]:
            if not title:
                self.calendars[calendar]["title"] = self.calendars[calendar][
                    "dir"
                ].get("displayname")
        self.title = "Tracklr"
        if title:
            self.title = title
        if "title" in self.calendars[calendar]:
            self.title = self.calendars[calendar]["title"]
        return self.title

    def get_subtitle(self, calendar, subtitle):
        """Handles title of the provided calendar.

        Title is optional in the configuration so default title is
        "Command-line Productivity Toolset".
        """
        self.subtitle = "Command-line Productivity Toolset"
        if subtitle:
            self.subtitle = subtitle
        if "subtitle" in self.calendars[calendar]:
            self.subtitle = self.calendars[calendar]["subtitle"]
        return self.subtitle

    def get_titles(self, calendar, title, subtitle):
        """Returns "title - subtitle" string.
        """
        cal = self.get_calendar_config(calendar)
        return "{} - {}".format(
            self.get_title(cal["name"], title),
            self.get_subtitle(cal["name"], subtitle),
        )

    def parse_tags(self, summary):
        """Parses given event summary and returns all #hashtags found.
        """
        tags = re.compile(r"#([a-zA-Z0-9_\.]+)")
        if isinstance(summary, str):
            return tags.findall(summary)
        if isinstance(summary, list):
            return tags.findall(" ".join(summary))

    def set_timezone(self, name):
        """Use this for feeds that use non-standard
        ``X-WR-TIMEZONE`` for timezones, or when a feed needs to apply specific
        timezone.

        TL;DR ``X-WR-TIMEZONE`` is NOT part of RFC 5545.

        For more info see:
        https://blog.jonudell.net/2011/10/17/x-wr-timezone-considered-harmful/
        """
        if self.calendars[name].get("x_wr_timezone", False) is not False:
            warnings.warn(WARN_CONFIG_TZ)
        timezone = self.calendars[name].get("timezone", False)
        if timezone is not False:
            if timezone is True:
                try:
                    tz = pytz.timezone(
                        self.calendars[name]["calendar"].get("X-WR-TIMEZONE")
                    )
                except:
                    tz = pytz.timezone("UTC")
            else:
                tz = pytz.timezone(self.calendars[name].get("timezone"))
        for component in self.calendars[name]["events"]:
            dtstart = component.get("DTSTART")
            dtend = component.get("DTEND")
            dtstamp = component.get("DTSTAMP")
            dtstart.dt = dtstart.dt.astimezone(tz)
            dtend.dt = dtend.dt.astimezone(tz)
            dtstamp.dt = dtstamp.dt.astimezone(tz)

    def get_pdf_output_file(self, file=None):
        """Return ``file`` is one is provided to input, or
        defaults to ``self.pdf_output_file = "report.pdf"``.
        """
        if file:
            return file
        return self.pdf_output_file

    def get_auth(self, username, password):
        """Returns ``HTTPBasicAuth`` for provided ``username`` and
        ``password``.
        """
        return HTTPBasicAuth(username, password)

    def get_feed(
        self,
        name,
        location,
        username=None,
        password=None,
        title=None,
        subtitle=None,
    ):
        """Loads calendar URL which can use BasicHTTPAuth.
        """
        self.calendars[name]["events"] = []
        try:
            if username and password:
                self.calendars[name]["auth"] = self.get_auth(
                    username,
                    password
                )
                resp = requests.get(
                    location,
                    auth=self.calendars[name]["auth"]
                )
            else:
                resp = requests.get(location)
            if resp.status_code == 200:
                self.calendars[name]["ics"] = resp.text
                self.calendars[name]["calendar"] = Calendar.from_ical(
                    self.calendars[name]["ics"]
                )
                for event in self.calendars[name]["calendar"].walk("vevent"):
                    self.calendars[name]["events"].append(event)
        except MissingSchema:
            try:
                self.calendars[name]["dir"] = Vdir(
                    os.path.expanduser(location)
                )
                for dir_event in self.calendars[name]["dir"].list():
                    event_ical = Calendar.from_ical(
                        self.calendars[name]["dir"].get(dir_event)
                    )
                    for event in event_ical.walk("vevent"):
                        self.calendars[name]["events"].append(event)
            except CollectionNotFoundError:
                self.log.warning("No calendar found at {}".format(location))

    def get_calendar(self, calendar):
        """Loads multiple calendars which can use BasicHTTPAuth.
        """
        cal = self.get_calendar_config(calendar)
        if "username" in cal and "password" in cal:
            try:
                self.get_feed(
                    cal["name"],
                    cal["location"],
                    cal["username"],
                    cal["password"]
                )
            except KeyError:
                self.get_feed(
                    cal["name"],
                    cal["url"],
                    cal["username"],
                    cal["password"]
                )
                warnings.warn(WARN_CONFIG_URL)
        else:
            try:
                self.get_feed(cal["name"], cal["location"])
            except KeyError:
                self.get_feed(cal["name"], cal["url"])
                warnings.warn(WARN_CONFIG_URL)

        if cal.get("x_wr_timezone", False) is not False:
            warnings.warn(WARN_CONFIG_TZ)
            self.set_timezone(cal["name"])

        if cal.get("timezone", False) is not False:
            self.set_timezone(cal["name"])

    def get_event_length(self, event):
        """Calculates length of an event.
        """
        return event["DTEND"].dt - event["DTSTART"].dt

    def get_event_date(self, event, format="%Y-%m-%d"):
        """Returns dates(s) of given event.
        """
        s = event["DTSTART"].dt
        e = event["DTEND"].dt
        if s.year == e.year and s.month == e.month and s.day == e.day:
            return s.strftime(format)
        else:
            return "{} - {}".format(s.strftime(format), e.strftime(format))

    def filter_event(self, event, date_pattern, include, exclude):
        """Decides whether the event should be included or excluded.
        """
        # abort if there is no summary
        try:
            summary = event["SUMMARY"].lower()
        except KeyError:
            return False
        # Filter by date pattern
        date = self.get_event_date(event)
        if date_pattern is not None and date_pattern not in date:
            return True
        # Filter by given include patterns
        if include is not None:
            for pattern in include:
                if pattern.lower() not in summary:
                    return True
            # Filter includes by tag match
            filter_out = False
            tags = self.parse_tags(include)
            for t in tags:
                if "#{}".format(t) not in summary:
                    filter_out = True
            if filter_out:
                return True
        # Filter by given exclude patterns
        if exclude is not None:
            for pattern in exclude:
                # first check for direct match
                if pattern.lower() in summary:
                    return True
            # Filter excludes by tag match
            filter_out = False
            tags = self.parse_tags(exclude)
            for t in tags:
                if "#{}".format(t) in summary:
                    filter_out = True
            if filter_out:
                return True
        return False

    def get_report(self, calendar, date_pattern, include, exclude):
        """Generates timesheet report in format:

            date, summary, description, hours
        """
        self.get_calendar(calendar)
        self.report = []
        self.report_html = []
        cal = self.get_calendar_config(calendar)

        for event in cal["events"]:

            if self.filter_event(event, date_pattern, include, exclude):
                continue

            try:
                summary = event["SUMMARY"].strip()
                tags = " ".join(
                    ["#{}".format(tag) for tag in self.parse_tags(summary)]
                )
            except KeyError:
                summary = ""
                tags = ""
            try:
                description = event["DESCRIPTION"].strip()
            except KeyError:
                description = ""

            date = self.get_event_date(event)
            lent = self.get_event_length(event)
            entry = (
                date,
                summary,
                description,
                lent.total_seconds() / 3600.0,
            )
            self.report.append(entry)

            entry_html = (
                date,
                str(summary).replace("\n", "<br />"),
                str(description).replace("\n", "<br />"),
                lent.total_seconds() / 3600.0,
                tags,
            )
            self.report_html.append(entry_html)

            self.total_seconds = self.total_seconds + lent.total_seconds()
            self.total_hours = self.total_seconds / 3600.0
        self.report = sorted(self.report)
        self.report_html = sorted(self.report_html)
        return self.report

    def get_tags(self, calendar, date_pattern, include, exclude):
        """Generates tags report in format:

            tag, hours
        """
        self.get_calendar(calendar)
        self.report = []
        cal = self.get_calendar_config(calendar)

        for event in cal["events"]:

            if self.filter_event(event, date_pattern, include, exclude):
                continue

            try:
                summary = event["SUMMARY"]
                tags = self.parse_tags(summary)
                tag_no = len(tags)
                if tag_no == 0:
                    tags = ["untagged"]
                    tag_no = 1
                if tags:
                    lent = self.get_event_length(event)
                    for t in tags:
                        if t in self.tags:
                            self.tags[t] += (
                                lent.total_seconds() / 3600.0
                            ) / float(tag_no)
                        else:
                            self.tags[t] = (
                                lent.total_seconds() / 3600.0
                            ) / float(tag_no)
                    self.total_seconds = (
                        self.total_seconds + lent.total_seconds()
                    )
                    self.total_hours = self.total_seconds / 3600.0
            except KeyError:
                self.log.debug("No summary found")

        if self.tags:
            for t in self.tags:
                entry = (t, "{:.1f}".format(self.tags[t]))
                self.report.append(entry)
        else:
            self.log.info("No tags found")

        self.report = sorted(self.report)
        self.report_html = self.report
        return self.report

    def get_parser(self, parser):
        """Returns parser with Tracklr's arguments:

        * ``-k`` ``--kalendar`` specify calendar to use.
          `default` calendar is used otherwise
        * ``-d`` ``--date`` date pattern eg. 2019, 2019-01
        * ``-t`` ``--title`` report title,
          or title from the config is used
        * ``-s`` ``--subtitle`` report subtitle,
          or subtitle from the config is used
        * ``-i`` ``--include`` include patterns. Tags need to be in quotes.
          Eg. -i @Tracklr "#v0.7"
        * ``-x`` ``--exclude`` exclude patterns. Tags need to be in quotes.
          Eg. -x "#tag"
        """
        parser.add_argument("-k", "--kalendar")
        parser.add_argument("-d", "--date")
        parser.add_argument("-t", "--title")
        parser.add_argument("-s", "--subtitle")
        parser.add_argument("-i", "--include", nargs="*")
        parser.add_argument("-x", "--exclude", nargs="*")
        return parser



import sys

from cliff.app import App
from cliff.commandmanager import CommandManager

from tracklr import Tracklr


class TracklrApp(App):
    def __init__(self):
        super(TracklrApp, self).__init__(
            description="Tracklr - Command-line Productivity Toolset",
            version=Tracklr.__version__,
            command_manager=CommandManager("tracklr"),
            deferred_help=True,
        )

    def initialize_app(self, argv):
        self.LOG.debug("initialize_app")

    def prepare_to_run_command(self, cmd):
        self.LOG.debug("prepare_to_run_command %s", cmd.__class__.__name__)

    def clean_up(self, cmd, result, err):
        self.LOG.debug("clean_up %s", cmd.__class__.__name__)
        if err:
            self.LOG.debug("got an error: %s", err)


def main(argv=sys.argv[1:]):
    myapp = TracklrApp()
    return myapp.run(argv)


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))



import logging

from cliff.lister import Lister
from tracklr import Tracklr


class Tag(Lister):

    log = logging.getLogger(__name__)

    tracklr = Tracklr()

    def take_action(self, parsed_args):
        """Generates report and logs total number of hours.
        """
        ts = self.tracklr.get_tags(
            parsed_args.kalendar,
            parsed_args.date,
            parsed_args.include,
            parsed_args.exclude,
        )

        self.tracklr.banner(
            parsed_args.kalendar,
            parsed_args.title,
            parsed_args.subtitle
        )

        self.log.info("\nTotal hours: {}\n".format(self.tracklr.total_hours))

        return (("Tag", "Hours"), ts)

    def get_description(self):
        return "creates tagged report"

    def get_parser(self, prog_name):
        parser = super(Tag, self).get_parser(prog_name)
        return self.tracklr.get_parser(parser)



import os
import logging

from cliff.command import Command

from tracklr import Tracklr
from tracklr.pdf import Pdf


class Init(Command):

    log = logging.getLogger(__name__)

    tracklr = Tracklr()

    def take_action(self, parsed_args):
        """Creates ``tracklr.yml`` and ``pdf.html`` out of defaults in:

        * *user config* directory
        * *local* directory
        """
        actions = {"config": self.init_config, "template": self.init_template}
        action = parsed_args.action
        self.tracklr.banner(
            parsed_args.kalendar,
            parsed_args.title,
            parsed_args.subtitle
        )
        self.log.info("\nInitializing {}\n".format(action))
        actions[action](parsed_args)

    def init_config(self, parsed_args):
        """Creates local or global config.
        """
        self.create_file_type(
            parsed_args, self.tracklr.config_file, self.tracklr.__config__
        )

    def init_template(self, parsed_args):
        """Creates local or global template.
        """
        self.create_file_type(
            parsed_args, self.tracklr.pdf_template_file, Pdf.__template__
        )

    def create_file_type(self, parsed_args, file_name, file_content):
        """Handles creation of given file.
        """
        if parsed_args.user_config_dir:
            self.log.info("user config dir")
            self.create_file(
                parsed_args, self.tracklr.global_path, file_name, file_content
            )
        else:
            self.log.info("current dir")
            self.create_file(
                parsed_args, self.tracklr.local_path, file_name, file_content
            )

    def create_file(self, parsed_args, file_path, file_name, file_content):
        """Creates desired file at given path with given content.
        """
        f = os.path.join(file_path, file_name)
        if os.path.isfile(f):
            self.log.info(
                "file {} already exists. " "nothing to do here...".format(f)
            )
        else:
            if not os.path.exists(file_path):
                self.log.info(
                    "directory {} does not exist. "
                    "creating...".format(file_path)
                )
                os.makedirs(file_path)
            self.log.info("file {} does not exist. " "creating...".format(f))
            fd = open(f, "w")
            fd.write(file_content)
            fd.close()

    def get_parser(self, prog_name):
        """Adds two arguments:

        * ``action`` - inits either ``config`` or ``template``
        * ``--user-config-dir`` - use this option to generate file in
          *user config* directory instead of current directory
        """
        parser = super(Init, self).get_parser(prog_name)
        parser.add_argument(
            "action",
            type=str,
            choices=["config", "template"],
            help="init config/template in the current directory",
        )
        parser.add_argument(
            "--user-config-dir",
            action="store_true",
            help="create config/template in {} "
            "instead of the current directory".format(
                self.tracklr.global_path
            ),
        )
        return parser

    def get_description(self):
        """initializes ``tracklr.yml`` and ``pdf.html``
        """
        return "initializes tracklr.yml and pdf.html"



#!/usr/bin/env python

PROJECT = "tracklr"

from tracklr import Tracklr

VERSION = Tracklr.__version__


from setuptools import setup, find_packages

try:
    long_description = open("README.rst", "rt").read()
except IOError:
    long_description = ""

setup(
    name=PROJECT,
    version=VERSION,
    description="Tracklr - Command-line Productivity Toolset",
    long_description=long_description,
    author="Marek Kuziel",
    author_email="marek@kuziel.info",
    url="https://gitlab.com/markuz/tracklr",
    download_url="https://gitlab.com/markuz/tracklr/-/archive/master/tracklr-master.tar.bz2",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Intended Audience :: Other Audience",
        "Environment :: Console",
    ],
    platforms=["Any"],
    scripts=[],
    provides=[],
    install_requires=[
        "appdirs",
        "cliff",
        "icalendar",
        "jinja2",
        "pyyaml",
        "requests",
        "xhtml2pdf",
    ],
    namespace_packages=[],
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        "console_scripts": ["tracklr = tracklr.main:main"],
        "tracklr": [
            "ls = tracklr.ls:Ls",
            "tag = tracklr.tag:Tag",
            "pdf = tracklr.pdf:Pdf",
            "init = tracklr.init:Init",
            "show = tracklr.show:Show",
        ],
    },
    zip_safe=False,
)



import logging

from cliff.command import Command
from tracklr import Tracklr


class Show(Command):

    log = logging.getLogger(__name__)

    tracklr = Tracklr()

    def take_action(self, parsed_args):
        """Show configuration details.
        """
        self.tracklr.banner(
            parsed_args.kalendar,
            parsed_args.title,
            parsed_args.subtitle
        )

        self.log.info("\nCalendars:\n")

        calendars = []
        for cal in self.tracklr.calendars:
            self.log.info(
                "* {} | {} | {}\n".format(
                    cal,
                    self.tracklr.get_title(cal, parsed_args.title),
                    self.tracklr.calendars[cal]["location"],
                )
            )

    def get_description(self):
        return "display info about the current instance"

    def get_parser(self, prog_name):
        parser = super(Show, self).get_parser(prog_name)
        return self.tracklr.get_parser(parser)

